/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.email

import cats.data.*

/** Container for an email message.
  *
  * @todo
  *   Support named addresses (`"Jane Doe <j.doe@example.com>"`) in the header fields.
  *
  * @param from
  *   The address *from* which the email is sent (*read* "the sender").
  * @param to
  *   A list of email addresses *to* which the email shall be sent. The list must contain at least one entry!
  * @param cc
  *   A list of email addresses which shall receive a copy of the email. This list may be empty.
  * @param bcc
  *   A list of email addresses which shall receive a blind copy of the email. This list may be empty.
  * @param subject
  *   A short descriptive message describing the subject of the email.
  * @param textBody
  *   The body of the email in plaintext format.
  */
final case class EmailMessage(
    from: FromAddress,
    to: NonEmptyList[ToAddress],
    cc: List[ToAddress],
    bcc: List[ToAddress],
    subject: SubjectLine,
    textBody: TextBody
)
