/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.email

import cats.kernel.Eq
import cats.syntax.all.*
import de.smederee.email.Generators.given
import jakarta.mail.Message.RecipientType

import munit.*

import org.scalacheck.*
import org.scalacheck.Prop.*

import scala.jdk.CollectionConverters.*

final class SimpleJavaMailMiddlewareHelpersTest extends ScalaCheckSuite {
    import SimpleJavaMailMiddleware.helpers.*

    given Eq[RecipientType] = Eq.fromUniversalEquals

    property("EmailMessage.toEmail must create correct Email instances") {
        forAll { (message: EmailMessage) =>
            val email = message.toEmail
            assertEquals(email.getFromRecipient.getAddress, message.from.toString)
            assertEquals(email.getSubject, message.subject.toString)
            assertEquals(email.getPlainText, message.textBody.toString)
            val recipients = email.getRecipients.asScala.toList
            message.to.toList.foreach { address =>
                val r = recipients.find(r => r.getAddress === address.toString && r.getType === RecipientType.TO)
                assert(r.nonEmpty)
            }
            message.cc.foreach { address =>
                val r = recipients.find(r => r.getAddress === address.toString && r.getType === RecipientType.CC)
                assert(r.nonEmpty)
            }
            message.bcc.foreach { address =>
                val r = recipients.find(r => r.getAddress === address.toString && r.getType === RecipientType.BCC)
                assert(r.nonEmpty)
            }
        }
    }

}
