/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.html

import com.comcast.ip4s.Host
import com.comcast.ip4s.Port
import org.http4s.Uri

/** A wrapper for bundling values to configure a possible "external url mapping", the idea is that a service might be
  * listening on a server and port which is behind a reverse proxy. The value configured here may have a multitude of
  * applications generation of valid urls and adjusting of CSRF protection only being examples.
  *
  * @param host
  *   The official hostname of the service which could be used for the CSRF protection, generation of links in e-mails
  *   etc.
  * @param path
  *   A possible path prefix that will be prepended to any paths used in link generation.
  * @param port
  *   The port number which defaults to the port the service is listening on. Please note that this is also relevant for
  *   CSRF protection! It should not be defined if the service is running behind a reverse proxy listening on the
  *   standard port for the given URL scheme (http/https).
  * @param scheme
  *   The URL scheme which is used for links and will also determine if cookies will have the secure flag enabled.
  */
final case class ExternalUrlConfiguration(host: Host, path: Option[Uri], port: Option[Port], scheme: Uri.Scheme)
