/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.hub

import de.smederee.tickets.Project
import de.smederee.tickets.ProjectDescription
import de.smederee.tickets.ProjectName
import de.smederee.tickets.ProjectOwner
import de.smederee.tickets.ProjectOwnerId
import de.smederee.tickets.ProjectOwnerName

/** A type class for converting types into each other.
  *
  * @tparam FROM
  *   The type from which a conversion shall be done.
  * @tparam TO
  *   The target type of the conversion.
  */
trait RelatedTypesConverter[FROM, TO] {
    extension (from: FROM) {

        /** Create an instance of the desired type.
          *
          * @return
          *   An instance of the desired type based on the data of the source type.
          */
        def convert: TO
    }
}

object RelatedTypesConverter {
    given RelatedTypesConverter[Account, ProjectOwner] with {
        extension (from: Account) {
            override def convert: ProjectOwner =
                ProjectOwner(
                    uid = ProjectOwnerId.fromUserId(from.uid),
                    name = ProjectOwnerName.fromUsername(from.name),
                    email = from.email
                )
        }
    }

    given RelatedTypesConverter[VcsRepository, Project] with {
        extension (from: VcsRepository) {
            override def convert: Project =
                Project(
                    owner = ProjectOwner(ProjectOwnerId.fromUserId(from.owner.uid), from.owner.name, from.owner.email),
                    name = ProjectName(from.name.toString),
                    description = from.description.map(descr => ProjectDescription(descr.toString)),
                    isPrivate = from.isPrivate
                )
        }
    }
}
