/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.tickets

import java.util.UUID

import cats.effect.*
import de.smederee.email.EmailAddress
import de.smederee.i18n.LanguageCode
import de.smederee.security.UserId
import de.smederee.security.Username
import doobie.*
import doobie.implicits.*
import doobie.postgres.implicits.*
import org.slf4j.LoggerFactory

final class DoobieTicketServiceApi[F[_]: Sync](tx: Transactor[F]) extends TicketServiceApi[F] {
    private val log = LoggerFactory.getLogger(getClass)

    given LogHandler[F] = Slf4jLogHandler.createLogHandler[F](log)

    given Meta[EmailAddress] = Meta[String].timap(EmailAddress.apply)(_.toString)
    given Meta[LanguageCode] = Meta[String].timap(LanguageCode.apply)(_.toString)
    given Meta[UserId]       = Meta[UUID].timap(UserId.apply)(_.toUUID)
    given Meta[Username]     = Meta[String].timap(Username.apply)(_.toString)

    override def createOrUpdateUser(user: TicketsUser): F[Int] =
        sql"""INSERT INTO tickets.users (uid, name, email, language, created_at, updated_at)
          VALUES (
            ${user.uid},
            ${user.name},
            ${user.email},
            ${user.language},
            NOW(),
            NOW()
          ) ON CONFLICT (uid) DO UPDATE SET
            name = EXCLUDED.name,
            email = EXCLUDED.email,
            language = EXCLUDED.language,
            updated_at = EXCLUDED.updated_at""".update.run.transact(tx)

    override def deleteUser(uid: UserId): F[Int] =
        sql"""DELETE FROM tickets.users WHERE uid = $uid""".update.run.transact(tx)
}
