/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.tickets

import cats.*
import de.smederee.email.EmailAddress
import de.smederee.i18n.LanguageCode
import de.smederee.security.UserId
import de.smederee.security.Username

/** A user of the tickets service.
  *
  * @param uid
  *   The unique ID of the user.
  * @param name
  *   A unique name which can be used for login and to identify the user.
  * @param email
  *   The email address of the user which must also be unique.
  * @param language
  *   The language code of the users preferred language.
  */
final case class TicketsUser(uid: UserId, name: Username, email: EmailAddress, language: Option[LanguageCode])

object TicketsUser {
    given Eq[TicketsUser] = Eq.fromUniversalEquals
}
