/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.tickets.config

import pureconfig.*

/** Configuration specifying the database access.
  *
  * @param driver
  *   The class name of the JDBC driver to be used.
  * @param url
  *   The JDBC connection URL **without** username and password.
  * @param user
  *   The username (login) needed to authenticate against the database.
  * @param pass
  *   The password needed to authenticate against the database.
  */
final case class DatabaseConfig(driver: String, url: String, user: String, pass: String)

object DatabaseConfig {
    given ConfigReader[DatabaseConfig] = ConfigReader.forProduct4("driver", "url", "user", "pass")(DatabaseConfig.apply)
}
