/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.hub

import java.nio.charset.StandardCharsets

import cats.syntax.all.*
import de.smederee.security.*

import munit.*

import org.scalacheck.*
import org.scalacheck.Prop.*

final class SessionIdTest extends ScalaCheckSuite {
    private val genKey: Gen[PrivateKey] =
        Gen.nonEmptyListOf(Gen.alphaNumChar).map(cs => PrivateKey(cs.mkString.getBytes(StandardCharsets.UTF_8)))
    given Arbitrary[PrivateKey]              = Arbitrary(genKey)
    private val genSessionId: Gen[SessionId] = Gen.delay(SessionId.generate)
    given Arbitrary[SessionId]               = Arbitrary(genSessionId)

    property("from must parse ids created with generate") {
        forAll { (id: SessionId) =>
            val obtained = SessionId.from(id.toString)
            assert(obtained.map(_ === id).getOrElse(false), s"$obtained not parsed into expectd $id")
        }
    }

    property("from must not parse random garbage") {
        forAll { (garbage: String) =>
            assert(SessionId.from(garbage).isEmpty)
        }
    }

    test("generate must produce sufficiently random session ids") {
        val builder = Vector.newBuilder[String]
        (0 until 15000000).foreach { _ =>
            builder.addOne(SessionId.generate.toString)
        }
        val ids = builder.result
        assertEquals(ids.distinct.length, ids.length, "Duplicate session id found!")
    }
}
