/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.ssh

import munit.*

final class SshKeyTypeTest extends FunSuite {
    test("from must work for ssh-dsa") {
        val input = scala.io.Source
            .fromInputStream(
                getClass().getClassLoader().getResourceAsStream("de/smederee/ssh/test-ssh-dsa.pub"),
                "UTF-8"
            )
            .getLines()
            .mkString
        assertEquals(
            SshKeyType.from(input),
            Some(SshKeyType.SshDsa),
            s"Incorrect key type extracted from: $input"
        )
    }

    test("from must work for ssh-ecdsa") {
        val input = scala.io.Source
            .fromInputStream(
                getClass().getClassLoader().getResourceAsStream("de/smederee/ssh/test-ssh-ecdsa.pub"),
                "UTF-8"
            )
            .getLines()
            .mkString
        assertEquals(
            SshKeyType.from(input),
            Some(SshKeyType.SshEcDsa),
            s"Incorrect key type extracted from: $input"
        )
    }

    test("from must work for ssh-ed25519") {
        val input = scala.io.Source
            .fromInputStream(
                getClass().getClassLoader().getResourceAsStream("de/smederee/ssh/test-ssh-ed25519.pub"),
                "UTF-8"
            )
            .getLines()
            .mkString
        assertEquals(
            SshKeyType.from(input),
            Some(SshKeyType.SshEd25519),
            s"Incorrect key type extracted from: $input"
        )
    }

    test("from must work for ssh-rsa") {
        val input = scala.io.Source
            .fromInputStream(
                getClass().getClassLoader().getResourceAsStream("de/smederee/ssh/test-ssh-rsa.pub"),
                "UTF-8"
            )
            .getLines()
            .mkString
        assertEquals(
            SshKeyType.from(input),
            Some(SshKeyType.SshRsa),
            s"Incorrect key type extracted from: $input"
        )
    }

}
