/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.tickets

import de.smederee.tickets.Generators.*

import munit.*

import org.scalacheck.*
import org.scalacheck.Prop.*

final class MilestoneTitleTest extends ScalaCheckSuite {
    given Arbitrary[MilestoneTitle] = Arbitrary(genMilestoneTitle)

    test("MilestoneTitle.from must fail on empty input") {
        assertEquals(MilestoneTitle.from(""), None)
    }

    property("MilestoneTitle.from must fail on too long input") {
        forAll { (input: String) =>
            if (input.length > MilestoneTitle.MaxLength)
                assertEquals(MilestoneTitle.from(input), None)
        }
    }

    property("MilestoneTitle.from must succeed on valid input") {
        forAll { (label: MilestoneTitle) =>
            val input = label.toString
            assertEquals(MilestoneTitle.from(input), Option(label))
        }
    }
}
