/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.i18n

import java.util.Locale

import munit.*

import org.scalacheck.*
import org.scalacheck.Prop.*

final class LanguageCodeTest extends ScalaCheckSuite {
    val genLocale: Gen[Locale] = Gen.oneOf(Locale.getAvailableLocales.toList.filter(_.getISO3Language.nonEmpty))
    given Arbitrary[Locale]    = Arbitrary(genLocale)

    val genLanguageCode: Gen[LanguageCode] = genLocale.map(_.getISO3Language).filter(_.nonEmpty).map(LanguageCode.apply)
    given Arbitrary[LanguageCode]          = Arbitrary(genLanguageCode)

    property("from must work for correct language codes") {
        forAll { (locale: Locale) =>
            LanguageCode.from(locale.getISO3Language) match {
                case None               => fail(s"No LanguageCode created from given input (locale: $locale)!")
                case Some(languageCode) => assertEquals(languageCode.toString, locale.getISO3Language)
            }
        }
    }

    property("toLocale must return correct locales") {
        forAll { (code: LanguageCode) =>
            val expectedLocale = Locale.forLanguageTag(code.toString)
            assertEquals(code.toLocale, expectedLocale)
        }
    }
}
