/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.security

import cats.*

opaque type PasswordHash = String
object PasswordHash {
    given Eq[PasswordHash] = Eq.fromUniversalEquals

    /** Create an instance of PasswordHash from the given String type.
      *
      * @param source
      *   An instance of type String which will be returned as a PasswordHash.
      * @return
      *   The appropriate instance of PasswordHash.
      */
    def apply(source: String): PasswordHash = source

    /** Try to create an instance of PasswordHash from the given String.
      *
      * @param source
      *   A String that should fulfil the requirements to be converted into a PasswordHash.
      * @return
      *   An option to the successfully converted PasswordHash.
      */
    def from(source: String): Option[PasswordHash] =
        Option(source).map(_.trim.nonEmpty) match {
            case Some(true) => Option(source.trim)
            case _          => None
        }
}
