/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.security

opaque type PrivateKey = Array[Byte]
object PrivateKey {

    /** Create an instance of PrivateKey from the given Array[Byte] type.
      *
      * @param source
      *   An instance of type Array[Byte] which will be returned as a PrivateKey.
      * @return
      *   The appropriate instance of PrivateKey.
      */
    def apply(source: Array[Byte]): PrivateKey = source

    /** Try to create an instance of PrivateKey from the given Array[Byte].
      *
      * @param source
      *   A Array[Byte] that should fulfil the requirements to be converted into a PrivateKey.
      * @return
      *   An option to the successfully converted PrivateKey.
      */
    def from(source: Array[Byte]): Option[PrivateKey] =
        if (source.isEmpty: Boolean)
            None
        else
            Option(source)

    extension (key: PrivateKey) {

        /** Return the corresponding array of bytes for the private key.
          *
          * @return
          *   An array of bytes representing the private key.
          */
        def toArray: Array[Byte] = key

    }
}
