/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.security

import java.util.UUID

import cats.syntax.all.*

import munit.*

import org.scalacheck.*
import org.scalacheck.Prop.*

final class UserIdTest extends ScalaCheckSuite {
    private val genUUID: Gen[UUID] = Gen.delay(UUID.randomUUID)
    given Arbitrary[UUID]          = Arbitrary(genUUID)

    property("UserId.fromString must fail on invalid input") {
        forAll { (input: String) =>
            assert(UserId.fromString(input).isLeft)
        }
    }

    property("UserId.fromString must succeed on valid input") {
        forAll { (uuid: UUID) =>
            assert(UserId.fromString(uuid.toString).isRight)
        }
    }

    property("Eq must hold") {
        forAll { (id1: UUID, id2: UUID) =>
            val uid1 = UserId(id1)
            val uid2 = UserId(id2)

            assert(uid1 === uid1, "Identical user ids must be considered equal!")
            assert(uid1 =!= uid2, "Different user ids must not be considered equal!")
        }
    }
}
